@@ -21,6 +21,7 @@
 
 #include "gd-main-box-child.h"
 #include "gd-main-icon-box-child.h"
+#include "gd-main-icon-box-icon.h"
 
 #include <gio/gio.h>
 #include <glib.h>
@@ -50,6 +51,21 @@ G_DEFINE_TYPE_WITH_CODE (GdMainIconBoxChild, gd_main_icon_box_child, GTK_TYPE_FL
                          G_ADD_PRIVATE (GdMainIconBoxChild)
                          G_IMPLEMENT_INTERFACE (GD_TYPE_MAIN_BOX_CHILD, gd_main_box_child_interface_init))
 
+static gboolean
+gd_main_icon_box_child_bind_text_to_visible (GBinding *binding,
+                                             const GValue *from_value,
+                                             GValue *to_value,
+                                             gpointer user_data)
+{
+  gboolean to_visible;
+  const gchar *from_text;
+
+  from_text = g_value_get_string (from_value);
+  to_visible = from_text != NULL && from_text[0] != '\0' ? TRUE : FALSE;
+  g_value_set_boolean (to_value, to_visible);
+  return TRUE;
+}
+
 static void
 gd_main_icon_box_check_button_toggled (GdMainIconBoxChild *self)
 {
@@ -68,6 +84,106 @@ gd_main_icon_box_check_button_toggled (GdMainIconBoxChild *self)
     gtk_flow_box_unselect_child (GTK_FLOW_BOX (parent), GTK_FLOW_BOX_CHILD (self));
 }
 
+static void
+gd_main_icon_box_child_update_layout (GdMainIconBoxChild *self)
+{
+  GdMainIconBoxChildPrivate *priv;
+  GtkWidget *grid;
+  GtkWidget *icon;
+  GtkWidget *overlay;
+
+  priv = gd_main_icon_box_child_get_instance_private (self);
+
+  gtk_container_foreach (GTK_CONTAINER (self), (GtkCallback) gtk_widget_destroy, NULL);
+
+  grid = gtk_grid_new ();
+  gtk_widget_set_valign (grid, GTK_ALIGN_CENTER);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
+  gtk_container_add (GTK_CONTAINER (self), grid);
+
+  overlay = gtk_overlay_new ();
+  gtk_container_add (GTK_CONTAINER (grid), overlay);
+
+  icon = gd_main_icon_box_icon_new (priv->item);
+  gtk_widget_set_hexpand (icon, TRUE);
+  gtk_container_add (GTK_CONTAINER (overlay), icon);
+
+  if (gd_main_box_item_get_pulse (priv->item))
+    {
+      GtkWidget *spinner;
+
+      spinner = gtk_spinner_new ();
+      gtk_widget_set_halign (spinner, GTK_ALIGN_CENTER);
+      gtk_widget_set_size_request (spinner, 32, 32);
+      gtk_widget_set_valign (spinner, GTK_ALIGN_CENTER);
+      gtk_spinner_start (GTK_SPINNER (spinner));
+      gtk_overlay_add_overlay (GTK_OVERLAY (overlay), spinner);
+    }
+
+  priv->check_button = gtk_check_button_new ();
+  gtk_widget_set_can_focus (priv->check_button, FALSE);
+  gtk_widget_set_halign (priv->check_button, GTK_ALIGN_END);
+  gtk_widget_set_valign (priv->check_button, GTK_ALIGN_END);
+  gtk_widget_set_no_show_all (priv->check_button, TRUE);
+  g_object_bind_property (self, "selection-mode", priv->check_button, "visible", G_BINDING_SYNC_CREATE);
+  gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->check_button);
+  g_signal_connect_swapped (priv->check_button,
+                            "toggled",
+                            G_CALLBACK (gd_main_icon_box_check_button_toggled),
+                            self);
+
+  if (priv->show_primary_text)
+    {
+      GtkWidget *primary_label;
+
+      primary_label = gtk_label_new (NULL);
+      gtk_widget_set_no_show_all (primary_label, TRUE);
+      gtk_label_set_ellipsize (GTK_LABEL (primary_label), PANGO_ELLIPSIZE_MIDDLE);
+      g_object_bind_property (priv->item, "primary-text", primary_label, "label", G_BINDING_SYNC_CREATE);
+      g_object_bind_property_full (priv->item,
+                                   "primary-text",
+                                   primary_label,
+                                   "visible",
+                                   G_BINDING_SYNC_CREATE,
+                                   gd_main_icon_box_child_bind_text_to_visible,
+                                   NULL,
+                                   NULL,
+                                   NULL);
+      gtk_container_add (GTK_CONTAINER (grid), primary_label);
+    }
+
+  if (priv->show_secondary_text)
+    {
+      GtkStyleContext *context;
+      GtkWidget *secondary_label;
+
+      secondary_label = gtk_label_new (NULL);
+      gtk_widget_set_no_show_all (secondary_label, TRUE);
+      gtk_label_set_ellipsize (GTK_LABEL (secondary_label), PANGO_ELLIPSIZE_END);
+      context = gtk_widget_get_style_context (secondary_label);
+      gtk_style_context_add_class (context, "dim-label");
+      g_object_bind_property (priv->item, "secondary-text", secondary_label, "label", G_BINDING_SYNC_CREATE);
+      g_object_bind_property_full (priv->item,
+                                   "secondary-text",
+                                   secondary_label,
+                                   "visible",
+                                   G_BINDING_SYNC_CREATE,
+                                   gd_main_icon_box_child_bind_text_to_visible,
+                                   NULL,
+                                   NULL,
+                                   NULL);
+      gtk_container_add (GTK_CONTAINER (grid), secondary_label);
+    }
+
+  gtk_widget_show_all (grid);
+}
+
+static void
+gd_main_icon_box_child_notify_pulse (GdMainIconBoxChild *self)
+{
+  gd_main_icon_box_child_update_layout (self);
+}
+
 static GdMainBoxItem *
 gd_main_icon_box_child_get_item (GdMainBoxChild *child)
 {
@@ -132,9 +248,21 @@ gd_main_icon_box_child_set_item (GdMainIconBoxChild *self, GdMainBoxItem *item)
 
   priv = gd_main_icon_box_child_get_instance_private (self);
 
+  if (priv->item != NULL)
+    g_signal_handlers_disconnect_by_func (priv->item, gd_main_icon_box_child_notify_pulse, self);
+
   if (!g_set_object (&priv->item, item))
     return;
 
+  if (priv->item != NULL)
+    {
+      g_signal_connect_object (priv->item,
+                               "notify::pulse",
+                               G_CALLBACK (gd_main_icon_box_child_notify_pulse),
+                               self,
+                               G_CONNECT_SWAPPED);
+    }
+
   g_object_notify (G_OBJECT (self), "item");
   gtk_widget_queue_draw (GTK_WIDGET (self));
 }
@@ -164,74 +292,6 @@ gd_main_icon_box_child_set_selection_mode (GdMainIconBoxChild *self, gboolean se
   gtk_widget_queue_draw (GTK_WIDGET (self));
 }
 
-static void
-gd_main_icon_box_child_update_layout (GdMainIconBoxChild *self)
-{
-  GdMainIconBoxChildPrivate *priv;
-  GtkWidget *grid;
-  GtkWidget *image;
-  GtkWidget *overlay;
-  cairo_surface_t *icon;
-
-  priv = gd_main_icon_box_child_get_instance_private (self);
-
-  gtk_container_foreach (GTK_CONTAINER (self), (GtkCallback) gtk_widget_destroy, NULL);
-
-  grid = gtk_grid_new ();
-  gtk_widget_set_halign (grid, GTK_ALIGN_CENTER);
-  gtk_widget_set_valign (grid, GTK_ALIGN_CENTER);
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
-  gtk_container_add (GTK_CONTAINER (self), grid);
-
-  overlay = gtk_overlay_new ();
-  gtk_widget_set_margin_bottom (overlay, 4);
-  gtk_container_add (GTK_CONTAINER (grid), overlay);
-
-  icon = gd_main_box_item_get_icon (priv->item);
-  image = gtk_image_new_from_surface (icon);
-  g_object_bind_property (priv->item, "icon", image, "surface", G_BINDING_DEFAULT);
-  gtk_container_add (GTK_CONTAINER (overlay), image);
-
-  priv->check_button = gtk_check_button_new ();
-  gtk_widget_set_can_focus (priv->check_button, FALSE);
-  gtk_widget_set_halign (priv->check_button, GTK_ALIGN_END);
-  gtk_widget_set_valign (priv->check_button, GTK_ALIGN_END);
-  gtk_widget_set_no_show_all (priv->check_button, TRUE);
-  g_object_bind_property (self, "selection-mode", priv->check_button, "visible", G_BINDING_SYNC_CREATE);
-  gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->check_button);
-  g_signal_connect_swapped (priv->check_button,
-                            "toggled",
-                            G_CALLBACK (gd_main_icon_box_check_button_toggled),
-                            self);
-
-  if (priv->show_primary_text)
-    {
-      GtkWidget *primary_label;
-
-      primary_label = gtk_label_new (NULL);
-      gtk_label_set_ellipsize (GTK_LABEL (primary_label), PANGO_ELLIPSIZE_MIDDLE);
-      gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
-      g_object_bind_property (priv->item, "primary-text", primary_label, "label", G_BINDING_SYNC_CREATE);
-      gtk_container_add (GTK_CONTAINER (grid), primary_label);
-    }
-
-  if (priv->show_secondary_text)
-    {
-      GtkStyleContext *context;
-      GtkWidget *secondary_label;
-
-      secondary_label = gtk_label_new (NULL);
-      gtk_label_set_ellipsize (GTK_LABEL (secondary_label), PANGO_ELLIPSIZE_END);
-      gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
-      context = gtk_widget_get_style_context (secondary_label);
-      gtk_style_context_add_class (context, "dim-label");
-      g_object_bind_property (priv->item, "secondary-text", secondary_label, "label", G_BINDING_SYNC_CREATE);
-      gtk_container_add (GTK_CONTAINER (grid), secondary_label);
-    }
-
-  gtk_widget_show_all (grid);
-}
-
 static void
 gd_main_icon_box_child_set_show_primary_text (GdMainIconBoxChild *self, gboolean show_primary_text)
 {
@@ -289,9 +349,6 @@ static void
 gd_main_icon_box_child_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
   GdMainIconBoxChild *self = GD_MAIN_ICON_BOX_CHILD (object);
-  GdMainIconBoxChildPrivate *priv;
-
-  priv = gd_main_icon_box_child_get_instance_private (self);
 
   switch (property_id)
     {
@@ -317,9 +374,6 @@ static void
 gd_main_icon_box_child_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
 {
   GdMainIconBoxChild *self = GD_MAIN_ICON_BOX_CHILD (object);
-  GdMainIconBoxChildPrivate *priv;
-
-  priv = gd_main_icon_box_child_get_instance_private (self);
 
   switch (property_id)
     {
@@ -347,14 +401,13 @@ gd_main_icon_box_child_init (GdMainIconBoxChild *self)
   GtkStyleContext *context;
 
   context = gtk_widget_get_style_context (GTK_WIDGET (self));
-  gtk_style_context_add_class (context, "content-tile");
+  gtk_style_context_add_class (context, "tile");
 }
 
 static void
 gd_main_icon_box_child_class_init (GdMainIconBoxChildClass *klass)
 {
   GObjectClass *oclass = G_OBJECT_CLASS (klass);
-  GtkWidgetClass *wclass = GTK_WIDGET_CLASS (klass);
 
   oclass->constructed = gd_main_icon_box_child_constructed;
   oclass->dispose = gd_main_icon_box_child_dispose;
