@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2015 – 2017 Red Hat, Inc.
+ * Copyright © 2015 – 2018 Red Hat, Inc.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,8 +48,8 @@ struct _PhotosImageView
   cairo_region_t *bbox_region;
   cairo_region_t *region;
   gboolean best_fit;
-  gdouble height;
-  gdouble width;
+  gdouble bbox_zoomed_height;
+  gdouble bbox_zoomed_width;
   gdouble x;
   gdouble x_scaled;
   gdouble y;
@@ -57,6 +57,8 @@ struct _PhotosImageView
   gdouble zoom;
   gdouble zoom_visible;
   gdouble zoom_visible_scaled;
+  gsize surface_memory_size;
+  guchar *surface_memory;
 };
 
 enum
@@ -126,9 +128,9 @@ photos_image_view_calculate_best_fit_zoom (PhotosImageView *self, gdouble *out_z
   GtkAllocation allocation;
   gdouble zoom;
   gdouble zoom_scaled = 1.0;
+  gint allocation_height_scaled;
+  gint allocation_width_scaled;
   gint scale_factor;
-  gint viewport_height_real;
-  gint viewport_width_real;
 
   if (!photos_image_view_has_allocation_and_extent (self))
     goto out;
@@ -137,13 +139,13 @@ photos_image_view_calculate_best_fit_zoom (PhotosImageView *self, gdouble *out_z
   bbox = *gegl_buffer_get_extent (self->buffer);
 
   scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (self));
-  viewport_height_real = allocation.height * scale_factor;
-  viewport_width_real = allocation.width * scale_factor;
+  allocation_height_scaled = allocation.height * scale_factor;
+  allocation_width_scaled = allocation.width * scale_factor;
 
-  if (bbox.height > viewport_height_real || bbox.width > viewport_width_real)
+  if (bbox.height > allocation_height_scaled || bbox.width > allocation_width_scaled)
     {
-      gdouble height_ratio = bbox.height / (gdouble) viewport_height_real;
-      gdouble width_ratio = bbox.width / (gdouble) viewport_width_real;
+      gdouble height_ratio = bbox.height / (gdouble) allocation_height_scaled;
+      gdouble width_ratio = bbox.width / (gdouble) allocation_width_scaled;
       gdouble max_ratio =  MAX (height_ratio, width_ratio);
 
       zoom_scaled = 1.0 / max_ratio;
@@ -314,7 +316,7 @@ photos_image_view_set_hadjustment_values (PhotosImageView *self)
   page_increment = page_size * 0.9;
   step_increment = page_size * 0.1;
 
-  upper = MAX ((gdouble) allocation.width, self->width);
+  upper = MAX ((gdouble) allocation.width, self->bbox_zoomed_width);
   g_return_if_fail (upper - page_size >= 0.0);
 
   value = self->x;
@@ -345,7 +347,7 @@ photos_image_view_set_vadjustment_values (PhotosImageView *self)
   page_increment = page_size * 0.9;
   step_increment = page_size * 0.1;
 
-  upper = MAX ((gdouble) allocation.height, self->height);
+  upper = MAX ((gdouble) allocation.height, self->bbox_zoomed_height);
   g_return_if_fail (upper - page_size >= 0.0);
 
   value = self->y;
@@ -366,9 +368,9 @@ photos_image_view_update (PhotosImageView *self)
   GeglRectangle bbox;
   GeglRectangle bbox_zoomed;
   GtkAllocation allocation;
+  gint allocation_height_scaled;
+  gint allocation_width_scaled;
   gint scale_factor;
-  gint viewport_height_real;
-  gint viewport_width_real;
 
   g_object_freeze_notify (G_OBJECT (self));
 
@@ -406,8 +408,8 @@ photos_image_view_update (PhotosImageView *self)
   bbox = *gegl_buffer_get_extent (self->buffer);
   scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (self));
 
-  viewport_height_real = allocation.height * scale_factor;
-  viewport_width_real = allocation.width * scale_factor;
+  allocation_height_scaled = allocation.height * scale_factor;
+  allocation_width_scaled = allocation.width * scale_factor;
 
   if (self->best_fit && self->zoom_animation == NULL)
     {
@@ -416,8 +418,8 @@ photos_image_view_update (PhotosImageView *self)
       bbox_zoomed.x = (gint) (self->zoom_visible_scaled * bbox.x + 0.5);
       bbox_zoomed.y = (gint) (self->zoom_visible_scaled * bbox.y + 0.5);
 
-      self->x_scaled = (bbox_zoomed.width - viewport_width_real) / 2.0;
-      self->y_scaled = (bbox_zoomed.height - viewport_height_real) / 2.0;
+      self->x_scaled = (bbox_zoomed.width - allocation_width_scaled) / 2.0;
+      self->y_scaled = (bbox_zoomed.height - allocation_height_scaled) / 2.0;
     }
   else
     {
@@ -428,32 +430,32 @@ photos_image_view_update (PhotosImageView *self)
       bbox_zoomed.x = (gint) (self->zoom_visible_scaled * bbox.x + 0.5);
       bbox_zoomed.y = (gint) (self->zoom_visible_scaled * bbox.y + 0.5);
 
-      if (bbox_zoomed.width > viewport_width_real)
+      if (bbox_zoomed.width > allocation_width_scaled)
         {
           ratio_old = (self->x_scaled
                        + self->allocation_scaled_old.width / 2.0
                        - (gdouble) self->bbox_zoomed_old.x)
                       / (gdouble) self->bbox_zoomed_old.width;
-          self->x_scaled = ratio_old * (gdouble) bbox_zoomed.width - (viewport_width_real / 2.0);
-          self->x_scaled = CLAMP (self->x_scaled, 0.0, (gdouble) bbox_zoomed.width - viewport_width_real);
+          self->x_scaled = ratio_old * (gdouble) bbox_zoomed.width - (allocation_width_scaled / 2.0);
+          self->x_scaled = CLAMP (self->x_scaled, 0.0, (gdouble) bbox_zoomed.width - allocation_width_scaled);
         }
       else
         {
-          self->x_scaled = (bbox_zoomed.width - viewport_width_real) / 2.0;
+          self->x_scaled = (bbox_zoomed.width - allocation_width_scaled) / 2.0;
         }
 
-      if (bbox_zoomed.height > viewport_height_real)
+      if (bbox_zoomed.height > allocation_height_scaled)
         {
           ratio_old = (self->y_scaled
                        + self->allocation_scaled_old.height / 2.0
                        - (gdouble) self->bbox_zoomed_old.y)
                       / (gdouble) self->bbox_zoomed_old.height;
-          self->y_scaled = ratio_old * (gdouble) bbox_zoomed.height - (viewport_height_real / 2.0);
-          self->y_scaled = CLAMP (self->y_scaled, 0.0, (gdouble) bbox_zoomed.height - viewport_height_real);
+          self->y_scaled = ratio_old * (gdouble) bbox_zoomed.height - (allocation_height_scaled / 2.0);
+          self->y_scaled = CLAMP (self->y_scaled, 0.0, (gdouble) bbox_zoomed.height - allocation_height_scaled);
         }
       else
         {
-          self->y_scaled = (bbox_zoomed.height - viewport_height_real) / 2.0;
+          self->y_scaled = (bbox_zoomed.height - allocation_height_scaled) / 2.0;
         }
     }
 
@@ -463,8 +465,8 @@ photos_image_view_update (PhotosImageView *self)
   self->x = self->x_scaled / (gdouble) scale_factor;
   self->y = self->y_scaled / (gdouble) scale_factor;
 
-  self->height = (gdouble) bbox_zoomed.height / (gdouble) scale_factor;
-  self->width = (gdouble) bbox_zoomed.width / (gdouble) scale_factor;
+  self->bbox_zoomed_height = (gdouble) bbox_zoomed.height / (gdouble) scale_factor;
+  self->bbox_zoomed_width = (gdouble) bbox_zoomed.width / (gdouble) scale_factor;
 
   g_signal_handlers_block_by_func (self->hadjustment, photos_image_view_adjustment_value_changed, self);
   g_signal_handlers_block_by_func (self->vadjustment, photos_image_view_adjustment_value_changed, self);
@@ -530,14 +532,14 @@ static void
 photos_image_view_draw_node (PhotosImageView *self, cairo_t *cr, GdkRectangle *rect)
 {
   const Babl *format;
+  GeglAbyssPolicy buffer_flags;
   GeglRectangle roi;
   cairo_surface_t *surface = NULL;
-  g_autofree guchar *buf = NULL;
-  gint bpp;
   gint scale_factor;
   gint stride;
   gint64 end;
   gint64 start;
+  gsize surface_memory_size;
 
   g_return_if_fail (GEGL_IS_BUFFER (self->buffer));
   g_return_if_fail (self->zoom_visible > 0.0);
@@ -550,32 +552,48 @@ photos_image_view_draw_node (PhotosImageView *self, cairo_t *cr, GdkRectangle *r
   roi.width  = rect->width * scale_factor;
   roi.height = rect->height * scale_factor;
 
-  format = babl_format ("cairo-ARGB32");
   stride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32, roi.width);
-  buf = g_malloc0_n (stride, roi.height);
+
+  /* It's assumed that this multiplication won't overflow, because, if
+   * it did then there would be a lot more problems. eg., attempts to
+   * allocate memory for the entire top-level GtkWindow would blow up.
+   */
+  surface_memory_size = (gsize) roi.height * (gsize) stride;
+  g_return_if_fail (self->surface_memory_size >= surface_memory_size);
+
+  format = babl_format ("cairo-ARGB32");
+
+  if (self->zoom_animation == NULL)
+    buffer_flags = GEGL_ABYSS_NONE | GEGL_BUFFER_FILTER_AUTO;
+  else
+    buffer_flags = GEGL_ABYSS_NONE | GEGL_BUFFER_FILTER_NEAREST;
 
   start = g_get_monotonic_time ();
 
-  bpp = babl_format_get_bytes_per_pixel (format);
-  stride = bpp * roi.width;
   gegl_buffer_get (self->buffer,
                    &roi,
                    self->zoom_visible_scaled,
                    format,
-                   buf,
+                   self->surface_memory,
                    stride,
-                   GEGL_ABYSS_NONE);
+                   buffer_flags);
 
   end = g_get_monotonic_time ();
   photos_debug (PHOTOS_DEBUG_GEGL,
-                "PhotosImageView: Node Blit: %d, %d, %d×%d, %" G_GINT64_FORMAT,
+                "PhotosImageView: Node Blit: %d, %d, %d×%d, %.4f, %d, %" G_GINT64_FORMAT,
                 rect->x,
                 rect->y,
                 rect->width,
                 rect->height,
+                self->zoom_visible_scaled,
+                buffer_flags,
                 end - start);
 
-  surface = cairo_image_surface_create_for_data (buf, CAIRO_FORMAT_ARGB32, roi.width, roi.height, stride);
+  surface = cairo_image_surface_create_for_data (self->surface_memory,
+                                                 CAIRO_FORMAT_ARGB32,
+                                                 roi.width,
+                                                 roi.height,
+                                                 stride);
   cairo_surface_set_device_scale (surface, (gdouble) scale_factor, (gdouble) scale_factor);
   cairo_set_source_surface (cr, surface, rect->x, rect->y);
   cairo_paint (cr);
@@ -617,15 +635,38 @@ static void
 photos_image_view_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
   PhotosImageView *self = PHOTOS_IMAGE_VIEW (widget);
+  gint allocation_height_scaled;
+  gint allocation_width_scaled;
   gint scale_factor;
+  gint stride;
+  gsize surface_memory_size;
 
   GTK_WIDGET_CLASS (photos_image_view_parent_class)->size_allocate (widget, allocation);
 
   photos_image_view_update (self);
 
   scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (self));
-  self->allocation_scaled_old.height = allocation->height * scale_factor;
-  self->allocation_scaled_old.width = allocation->width * scale_factor;
+  allocation_height_scaled = allocation->height * scale_factor;
+  allocation_width_scaled = allocation->width * scale_factor;
+
+  stride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32, allocation_width_scaled);
+
+  /* It's assumed that this multiplication won't overflow, because, if
+   * it did then there would be a lot more problems. eg., attempts to
+   * allocate memory for the entire top-level GtkWindow would blow up.
+   */
+  surface_memory_size = (gsize) allocation_height_scaled * (gsize) stride;
+
+  if (self->surface_memory_size < surface_memory_size)
+    {
+      g_clear_pointer (&self->surface_memory, g_free);
+
+      self->surface_memory = (guchar *) g_malloc0 (surface_memory_size);
+      self->surface_memory_size = surface_memory_size;
+    }
+
+  self->allocation_scaled_old.height = allocation_height_scaled;
+  self->allocation_scaled_old.width = allocation_width_scaled;
 }
 
 
@@ -734,6 +775,7 @@ photos_image_view_finalize (GObject *object)
 
   g_clear_pointer (&self->bbox_region, (GDestroyNotify) cairo_region_destroy);
   g_clear_pointer (&self->region, (GDestroyNotify) cairo_region_destroy);
+  g_clear_pointer (&self->surface_memory, g_free);
 
   G_OBJECT_CLASS (photos_image_view_parent_class)->finalize (object);
 }
@@ -1098,8 +1140,8 @@ photos_image_view_set_node (PhotosImageView *self, GeglNode *node)
   self->allocation_scaled_old.height = 0;
   self->allocation_scaled_old.width = 0;
   self->best_fit = TRUE;
-  self->height = 0.0;
-  self->width = 0.0;
+  self->bbox_zoomed_height = 0.0;
+  self->bbox_zoomed_width = 0.0;
   self->x = 0.0;
   self->x_scaled = 0.0;
   self->y = 0.0;
