@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2016 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -81,6 +79,13 @@ photos_share_point_email_get_name (PhotosSharePoint *share_point)
 }
 
 
+static gboolean
+photos_share_point_email_is_search_criterion (PhotosFilterable *filterable)
+{
+  return FALSE;
+}
+
+
 static gboolean
 photos_share_point_email_needs_notification (PhotosSharePoint *share_point)
 {
@@ -92,6 +97,7 @@ static void
 photos_share_point_email_share_save_to_dir (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   PhotosSharePointEmail *self;
+  GAppLaunchContext *ctx = NULL;
   GError *error;
   GFile *file = NULL;
   GTask *task = G_TASK (user_data);
@@ -114,8 +120,10 @@ photos_share_point_email_share_save_to_dir (GObject *source_object, GAsyncResult
   escaped_path = g_uri_escape_string (path, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, FALSE);
   uri = g_strconcat ("mailto:?attach=", escaped_path, NULL);
 
+  ctx = photos_utils_new_app_launch_context_from_widget (NULL);
+
   error = NULL;
-  if (!photos_glib_app_info_launch_uri (self->default_app, uri, NULL, &error))
+  if (!photos_glib_app_info_launch_uri (self->default_app, uri, ctx, &error))
     {
       g_task_return_error (task, error);
       goto out;
@@ -127,6 +135,7 @@ photos_share_point_email_share_save_to_dir (GObject *source_object, GAsyncResult
   g_free (escaped_path);
   g_free (path);
   g_free (uri);
+  g_clear_object (&ctx);
   g_clear_object (&file);
   g_object_unref (task);
 }
@@ -271,6 +280,7 @@ static void
 photos_share_point_email_filterable_iface_init (PhotosFilterableInterface *iface)
 {
   iface->get_id = photos_share_point_email_get_id;
+  iface->is_search_criterion = photos_share_point_email_is_search_criterion;
 }
 
 
