@@ -1080,18 +1080,20 @@ photos_utils_get_version (void)
 }
 
 
-void
-photos_utils_launch_online_accounts (const gchar *account_id)
+gboolean
+photos_utils_launch_online_accounts (const gchar *account_id, GError **error)
 {
-  GApplication *app;
-  g_autoptr (GDBusActionGroup) control_center = NULL;
-  GDBusConnection *connection;
   GVariant *parameters;
+  g_autoptr (GApplication) control_center = NULL;
   g_auto (GVariantBuilder) panel_parameters = G_VARIANT_BUILDER_INIT (G_VARIANT_TYPE ("av"));
+  gboolean ret_val = FALSE;
+  const gchar *control_center_id = "org.gnome.ControlCenter";
 
-  app = g_application_get_default ();
-  connection = g_application_get_dbus_connection (app);
-  control_center = g_dbus_action_group_get (connection, "org.gnome.ControlCenter", "/org/gnome/ControlCenter");
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  control_center = g_application_new (control_center_id, G_APPLICATION_IS_LAUNCHER);
+  if (!g_application_register (control_center, NULL, error))
+    goto out;
 
   if (account_id != NULL && account_id[0] != '\0')
     {
@@ -1103,6 +1105,11 @@ photos_utils_launch_online_accounts (const gchar *account_id)
 
   parameters = g_variant_new ("(s@av)", "online-accounts", g_variant_builder_end (&panel_parameters));
   g_action_group_activate_action (G_ACTION_GROUP (control_center), "launch-panel", parameters);
+
+  ret_val = TRUE;
+
+ out:
+  return ret_val;
 }
 
 
